<?php

/**
 * Implements hook_menu().
 */
function cfrpreset_menu() {
  $items = array();
  $items['admin/structure/cfrpreset'] = array(
    'type' => MENU_NORMAL_ITEM,
    'access arguments' => array('administer cfrpreset'),
    'title' => 'Cfr plugin presets',
    /* @see _cfrpreset_overview_page() */
    'page callback' => '_cfrpreset_overview_page',
    'file' => 'cfrpreset.pages.overview.inc',
  );
  $items['admin/structure/cfrpreset/%_cfrpreset_interface'] = array(
    'type' => MENU_NORMAL_ITEM,
    'access arguments' => array('administer cfrpreset'),
    /* @see _cfrpreset_interface_page_title() */
    'title callback' => '_cfrpreset_interface_page_title',
    'title arguments' => array(3),
    /* @see _cfrpreset_interface_page() */
    'page callback' => '_cfrpreset_interface_page',
    'page arguments' => array(3),
    'file' => 'cfrpreset.pages.interface.inc',
  );
  $items['admin/structure/cfrpreset/%_cfrpreset_interface/list'] = array(
    'type' => MENU_DEFAULT_LOCAL_TASK,
    'title' => 'List',
    'weight' => -10,
  );
  $items['admin/structure/cfrpreset/%_cfrpreset_interface/code'] = array(
    'type' => MENU_LOCAL_TASK,
    'access arguments' => array('administer cfrpreset'),
    'title' => 'View code',
    /* @see _cfrpreset_interface_code_page() */
    'page callback' => '_cfrpreset_interface_code_page',
    'page arguments' => array(3),
    'file' => 'cfrpreset.pages.code.inc',
  );
  $items['admin/structure/cfrpreset/%_cfrpreset_interface/add'] = array(
    'type' => MENU_LOCAL_TASK | MENU_LOCAL_ACTION,
    'access arguments' => array('administer cfrpreset'),
    'title' => 'Add preset',
    'page callback' => 'drupal_get_form',
    /* @see _cfrpreset_preset_form() */
    'page arguments' => array('_cfrpreset_preset_form', 3),
    'file' => 'cfrpreset.pages.preset_form.inc',
  );
  $items['admin/structure/cfrpreset/%_cfrpreset_interface/preset/%_cfrpreset_preset'] = array(
    'type' => MENU_NORMAL_ITEM,
    'load arguments' => array(3),
    'access arguments' => array('administer cfrpreset'),
    /* @see _cfrpreset_preset_page_title() */
    'title callback' => '_cfrpreset_preset_page_title',
    'title arguments' => array(3, 5),
    'page callback' => 'drupal_get_form',
    /* @see _cfrpreset_preset_form() */
    'page arguments' => array('_cfrpreset_preset_form', 3, 5),
    'file' => 'cfrpreset.pages.preset_form.inc',
  );
  $preset_tabs = FALSE;
  if (module_exists('devel')) {
    $items['admin/structure/cfrpreset/%_cfrpreset_interface/preset/%_cfrpreset_preset/devel'] = array(
      'type' => MENU_LOCAL_TASK,
      'load arguments' => array(3),
      'access arguments' => array('administer cfrpreset'),
      'title' => 'Devel',
      /* @see _cfrpreset_preset_devel_page() */
      'page callback' => '_cfrpreset_preset_devel_page',
      'page arguments' => array(5),
      'file' => 'cfrpreset.pages.preset_devel.inc',
      'weight' => 10,
    );
    $preset_tabs = TRUE;
  }
  if ($preset_tabs) {
    $items['admin/structure/cfrpreset/%_cfrpreset_interface/preset/%_cfrpreset_preset/edit'] = array(
      'type' => MENU_DEFAULT_LOCAL_TASK,
      'title' => 'Edit',
      'weight' => -10,
    );
  }
  $items['admin/structure/cfrpreset/%_cfrpreset_interface/preset/%_cfrpreset_preset/delete'] = array(
    'type' => MENU_LOCAL_TASK,
    'context' => MENU_CONTEXT_INLINE,
    'load arguments' => array(3),
    'access arguments' => array('administer cfrpreset'),
    'title' => 'Delete',
    'page callback' => 'drupal_get_form',
    /* @see _cfrpreset_preset_delete_confirm_form() */
    'page arguments' => array('_cfrpreset_preset_delete_confirm_form', 5),
    'file' => 'cfrpreset.pages.preset_delete.inc',
  );
  return $items;
}

/**
 * Menu wildcard loader for '%_cfrpreset_interface'
 *
 * @param string $arg
 *
 * @return mixed
 */
function _cfrpreset_interface_load($arg) {
  $interface = str_replace('.', '\\', $arg);
  if (!_cfrpreset_valid_interface_name($interface)) {
    return NULL;
  }
  // At this point, $interface looks like a valid class name. But it could still
  // be a non-existing interface, and possibly something ridiculously long.
  // Avoid interface_exists(), because autoload can have side effects.
  return $interface;
}

/**
 * Menu wildcard loader for '%_cfrpreset_preset'
 *
 * @param string $arg
 * @param string $interface_arg
 *
 * @return object|null|\Drupal\cfrpreset\CfrPreset\CfrPresetInterface
 */
function _cfrpreset_preset_load($arg, $interface_arg) {
  $interface = _cfrpreset_interface_load($interface_arg);
  if (NULL === $interface) {
    return NULL;
  }
  return _cfrpreset_load_preset($interface, $arg);
}

/**
 * @param string $interface
 *
 * @return string
 */
function _cfrpreset_interface_page_title($interface) {
  return _cfrpreset_interface_get_title($interface);
}

/**
 * @param string $interface
 * @param object|\Drupal\cfrpreset\CfrPreset\CfrPresetInterface $preset
 *
 * @return string
 */
function _cfrpreset_preset_page_title(
  /** @noinspection PhpUnusedParameterInspection */ $interface,
  $preset
) {
  return $preset->label;
}

/**
 * @param object|\Drupal\cfrpreset\CfrPreset\CfrPresetInterface $record
 *
 * @return string
 */
function _cfrpreset_preset_get_path($record) {
  return _cfrpreset_interface_get_path($record->interface) . '/preset/' . $record->machine_name;
}

/**
 * @param string $interface
 *
 * @return string
 */
function _cfrpreset_interface_get_path($interface) {
  return 'admin/structure/cfrpreset/' . str_replace('\\', '.', $interface);
}
