<?php

/**
 * Form builder callback.
 *
 * @param array $form
 * @param array $form_state
 * @param string $interface
 * @param object|null|\Drupal\cfrpreset\CfrPreset\CfrPresetInterface $record
 *
 * @return array
 */
function _cfrpreset_preset_form(array $form, array &$form_state, $interface, $record = NULL) {

  $form['#record'] = $record;

  $form['identifier'] = array(
    '#type' => 'value',
    '#value' => NULL,
  );
  $form['interface'] = array(
    '#type' => 'value',
    '#value' => $interface,
  );
  $form['label'] = array(
    '#title' => t('Administrative title'),
    '#type' => 'textfield',
  );

  // Placeholder.
  $form['machine_name'] = array();

  // Placeholder.
  $form['plugin'] = array();

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#weight' => 5,
    /* @see _cfrpreset_preset_form_submit() */
    '#submit' => array('_cfrpreset_preset_form_submit'),
  );

  if (NULL !== $record) {
    $form['identifier']['#default_value'] = $record->identifier;
    $form['machine_name']['#default_value'] = $record->machine_name;
    $form['label']['#default_value'] = $record->label;

    $form['machine_name'] = array(
      '#type' => 'value',
      '#value' => $record->machine_name,
    );

    $conf = array(
      'id' => $record->plugin_id,
      'options' => $record->plugin_options,
    );

    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
      '#weight' => 15,
      /* @see _cfrpreset_preset_form_delete_submit() */
      '#submit' => array('_cfrpreset_preset_form_delete_submit'),
    );
  }
  else {
    $form['machine_name'] = array(
      '#title' => t('Machine name'),
      /* @see form_process_machine_name() */
      /* @see form_validate_machine_name() */
      '#type' => 'machine_name',
      '#machine_name' => array(
        'source' => array('label'),
        /* @see _cfrpreset_preset_form_machine_name_exists() */
        'exists' => '_cfrpreset_preset_form_machine_name_exists',
      ),
      // This is picked up by the 'exists' function.
      '#cfrpreset_interface' => $interface,
      '#required' => TRUE,
    );

    $conf = NULL;
  }

  if (isset($form_state['values']['plugin'])) {
    $conf = $form_state['values']['plugin'];
  }
  elseif (isset($form_state['input']['plugin'])) {
    $conf = $form_state['input']['plugin'];
  }

  $configurator = cfrplugin()->interfaceGetConfigurator($interface);
  $form['plugin'] = $configurator->confGetForm($conf, t('Plugin'));

  return $form;
}

/**
 * Form validation callback.
 *
 * @param array $form
 * @param array $form_state
 */
function _cfrpreset_preset_form_validate(array &$form, array &$form_state) {

  $values = $form_state['values'];
  if (!empty($form['#record'])) {
    /** @var object|\Drupal\cfrpreset\CfrPreset\CfrPresetInterface $record */
    $record = $form['#record'];
    // Check if record has changed in the database.
    if (array() !== _cfrpreset_preset_find_modifications($record)) {
      # form_set_error('', t('It appears that the preset was modified by someone else.'));
      # return;
    }
  }
  else {
    $record = _cfrpreset_create_stub($values['interface'], $values['machine_name']);
  }

  $record->label = $values['label'];
  $record->plugin_id = isset($values['plugin']['id']) ? $values['plugin']['id'] : NULL;
  $record->plugin_options = isset($values['plugin']['options']) ? $values['plugin']['options'] : NULL;

  $form_state['cfrpreset_preset_to_save'] = $record;
}

/**
 * Form submit callback.
 *
 * @param array $form
 * @param array $form_state
 */
function _cfrpreset_preset_form_submit(
  /** @noinspection PhpUnusedParameterInspection */ array &$form,
  array &$form_state
) {
  if (empty($form_state['cfrpreset_preset_to_save'])) {
    drupal_set_message('No record that could be saved.', 'error');
    return;
  }
  /** @var object|\Drupal\cfrpreset\CfrPreset\CfrPresetInterface $record */
  $record = $form_state['cfrpreset_preset_to_save'];
  $success = _cfrpreset_preset_save($record);
  if (FALSE === $success) {
    drupal_set_message('Failed to save the preset.', 'error');
  }
  elseif (SAVED_NEW === $success) {
    drupal_set_message('New preset created.');
    $form_state['redirect'] = _cfrpreset_preset_get_path($record);
  }
  elseif (SAVED_UPDATED === $success) {
    drupal_set_message('Preset updated.');
  }
}

/**
 * Form submit callback.
 *
 * @param array $form
 * @param array $form_state
 */
function _cfrpreset_preset_form_delete_submit(array &$form, array &$form_state) {
  $destination = array();
  if (isset($_GET['destination'])) {
    $destination = drupal_get_destination();
    unset($_GET['destination']);
  }
  $record = $form['#record'];
  $form_state['redirect'] = array(_cfrpreset_preset_get_path($record) . '/delete', array('query' => $destination));
}
